"use server";

import { createAuthorization } from "@/lib/dto/authorization";
import { getAuthorizeUrl } from "@/lib/oauth/authorize-url";
import { prisma } from "@/lib/prisma";

export async function handleAuthorizeAction(
  oauth: string,
  userId: string,
  clientId: string,
  scope: string,
) {
  // 检查客户端是否限制了允许的用户
  const client = await prisma.client.findUnique({
    where: { id: clientId },
    select: { allowedUsers: true },
  });

  if (!client) {
    throw new Error("应用不存在");
  }

  // 如果设置了允许的用户列表，检查当前用户是否在列表中
  if (client.allowedUsers.length > 0) {
    const user = await prisma.user.findUnique({
      where: { id: userId },
      select: { username: true },
    });

    if (!user || !client.allowedUsers.includes(user.username)) {
      throw new Error("您没有权限使用此应用");
    }
  }

  const oauthParams = new URLSearchParams(atob(oauth));
  const redirectUrl = getAuthorizeUrl(oauthParams);

  // 保存授权
  await createAuthorization({
    userId,
    clientId,
    scope,
    enabled: true, // 新授权默认启用
    lastUsedAt: new Date(), // 首次授权时间作为最后使用时间
  });

  return redirectUrl;
}
